@@ -34,9 +34,9 @@
 #   Eric Veach, Laurence Gonsalves, Matthew Springer, Craig Silverstein,
 #   Vladimir Rusinov
 
-"""This module is used to define and parse command line flags.
+"""This package is used to define and parse command line flags.
 
-This module defines a *distributed* flag-definition policy: rather than
+This package defines a *distributed* flag-definition policy: rather than
 an application having to define all flags in or near main(), each python
 module defines flags that are useful to it.  When one python module
 imports another, it gains access to the other's flags.  (This is
@@ -52,8 +52,11 @@ import getopt
 import os
 import re
 import sys
+import types
 import warnings
 
+import six
+
 from gflags import _helpers
 from gflags import argument_parser
 from gflags import exceptions
@@ -70,25 +73,19 @@ _helpers.disclaim_module_ids.add(id(sys.modules[__name__]))
 # suppress warning for whole block:
 # pylint: disable=invalid-name
 
-# Following aliases required because there is a lot of code that uses private
-# functions from here.
-# TODO(vrusinov): this code needs to be fixed and aliases needs to be removed.
-_GetModuleObjectAndName = _helpers.GetModuleObjectAndName
-_GetMainModule = _helpers.GetMainModule
-_DamerauLevenshtein = _helpers._DamerauLevenshtein  # pylint: disable=protected-access
-
 # Module exceptions:
-# TODO(vrusinov): these should all be renamed to *Error, e.g. DuplicateFlag
-# should be removed in favour of DuplicateFlagError.
-FlagsError = exceptions.FlagsError
+# TODO(vrusinov): these should all be renamed to *Error, e.g. IllegalFlagValue
+# should be removed in favour of IllegalFlagValueError.
+FlagsError = exceptions.Error
+Error = exceptions.Error
 CantOpenFlagFileError = exceptions.CantOpenFlagFileError
-DuplicateFlag = exceptions.DuplicateFlag
 DuplicateFlagError = exceptions.DuplicateFlagError
 DuplicateFlagCannotPropagateNoneToSwig = (
     exceptions.DuplicateFlagCannotPropagateNoneToSwig)
-IllegalFlagValue = exceptions.IllegalFlagValue
-UnrecognizedFlag = exceptions.UnrecognizedFlag
+IllegalFlagValue = exceptions.IllegalFlagValueError
+IllegalFlagValueError = exceptions.IllegalFlagValueError
 UnrecognizedFlagError = exceptions.UnrecognizedFlagError
+ValidationError = exceptions.ValidationError
 
 # Public functions:
 GetHelpWidth = _helpers.GetHelpWidth
@@ -127,10 +124,10 @@ FLAGS = FlagValues()
 # Flags validators
 
 
-def RegisterValidator(flag_name,
-                      checker,
-                      message='Flag validation failed',
-                      flag_values=FLAGS):
+def register_validator(flag_name,
+                       checker,
+                       message='Flag validation failed',
+                       flag_values=FLAGS):
   """Adds a constraint, which will be enforced during program execution.
 
   The constraint is validated when flags are initially parsed, and after each
@@ -144,34 +141,33 @@ def RegisterValidator(flag_name,
       output - Boolean.
         Must return True if validator constraint is satisfied.
         If constraint is not satisfied, it should either return False or
-          raise gflags_validators.Error(desired_error_message).
+          raise gflags.ValidationError(desired_error_message).
     message: Error text to be shown to the user if checker returns False.
-      If checker raises gflags_validators.Error, message from the raised
+      If checker raises gflags.ValidationError, message from the raised
         Error will be shown.
     flag_values: An optional FlagValues instance to validate against.
   Raises:
     AttributeError: If flag_name is not registered as a valid flag name.
   """
-  flag_values.AddValidator(gflags_validators.SimpleValidator(flag_name,
-                                                            checker,
-                                                            message))
+  v = gflags_validators.SingleFlagValidator(flag_name, checker, message)
+  _add_validator(flag_values, v)
 
 
-def Validator(flag_name, message='Flag validation failed', flag_values=FLAGS):
+def validator(flag_name, message='Flag validation failed', flag_values=FLAGS):
   """A function decorator for defining a flag validator.
 
   Registers the decorated function as a validator for flag_name, e.g.
 
-  @gflags.Validator('foo')
+  @gflags.validator('foo')
   def _CheckFoo(foo):
     ...
 
-  See RegisterValidator() for the specification of checker function.
+  See register_validator() for the specification of checker function.
 
   Args:
     flag_name: string, name of the flag to be checked.
     message: error text to be shown to the user if checker returns False.
-      If checker raises gflags_validators.Error, message from the raised
+      If checker raises gflags.ValidationError, message from the raised
         Error will be shown.
     flag_values: FlagValues
   Returns:
@@ -180,16 +176,84 @@ def Validator(flag_name, message='Flag validation failed', flag_values=FLAGS):
     AttributeError: if flag_name is not registered as a valid flag name.
   """
 
-  def Decorate(function):
-    RegisterValidator(flag_name, function,
-                      message=message,
-                      flag_values=flag_values)
+  def decorate(function):
+    register_validator(flag_name, function,
+                       message=message,
+                       flag_values=flag_values)
     return function
-  return Decorate
+  return decorate
 
 
-def MarkFlagAsRequired(flag_name, flag_values=FLAGS):
-  """Ensure that flag is not None during program execution.
+def register_multi_flags_validator(flag_names,
+                                   multi_flags_checker,
+                                   message='Flags validation failed',
+                                   flag_values=FLAGS):
+  """Adds a constraint to multiple flags.
+
+  The constraint is validated when flags are initially parsed, and after each
+  change of the corresponding flag's value.
+
+  Args:
+    flag_names: [str], a list of the flag names to be checked.
+    multi_flags_checker: callable, a function to validate the flag.
+        input - dictionary, with keys() being flag_names, and value for each key
+            being the value of the corresponding flag (string, boolean, etc).
+        output - Boolean. Must return True if validator constraint is satisfied.
+            If constraint is not satisfied, it should either return False or
+            raise gflags.ValidationError.
+    message: Error text to be shown to the user if checker returns False.
+        If checker raises gflags.ValidationError, message from the raised error
+        will be shown.
+    flag_values: An optional FlagValues instance to validate against.
+
+  Raises:
+    AttributeError: If a flag is not registered as a valid flag name.
+  """
+  v = gflags_validators.MultiFlagsValidator(
+      flag_names, multi_flags_checker, message)
+  _add_validator(flag_values, v)
+
+
+def multi_flags_validator(flag_names,
+                          message='Flag validation failed',
+                          flag_values=FLAGS):
+  """A function decorator for defining a multi-flag validator.
+
+  Registers the decorated function as a validator for flag_names, e.g.
+
+  @gflags.multi_flags_validator(['foo', 'bar'])
+  def _CheckFooBar(flags_dict):
+    ...
+
+  See register_multi_flags_validator() for the specification of checker
+  function.
+
+  Args:
+    flag_names: [str], a list of the flag names to be checked.
+    message: error text to be shown to the user if checker returns False.
+        If checker raises ValidationError, message from the raised
+        error will be shown.
+    flag_values: An optional FlagValues instance to validate against.
+
+  Returns:
+    A function decorator that registers its function argument as a validator.
+
+  Raises:
+    AttributeError: If a flag is not registered as a valid flag name.
+  """
+
+  def decorate(function):
+    register_multi_flags_validator(flag_names,
+                                   function,
+                                   message=message,
+                                   flag_values=flag_values)
+    return function
+
+  return decorate
+
+
+def mark_flag_as_required(flag_name, flag_values=FLAGS):
+  """Ensures that flag is not None during program execution.
 
   Registers a flag validator, which will follow usual validator rules.
   Important note: validator will pass for any non-None value, such as False,
@@ -198,7 +262,7 @@ def MarkFlagAsRequired(flag_name, flag_values=FLAGS):
   It is recommended to call this method like this:
 
     if __name__ == '__main__':
-      MarkFlagAsRequired('your_flag_name')
+      gflags.mark_flag_as_required('your_flag_name')
       app.run()
 
   Because validation happens at app.run() we want to ensure required-ness
@@ -212,25 +276,25 @@ def MarkFlagAsRequired(flag_name, flag_values=FLAGS):
   Raises:
     AttributeError: if flag_name is not registered as a valid flag name.
   """
-  if flag_values.GetFlag(flag_name).default is not None:
+  if flag_values[flag_name].default is not None:
     # TODO(vrusinov): Turn this warning into an exception.
     warnings.warn(
         'Flag %s has a non-None default value; therefore, '
-        'MarkFlagAsRequired will pass even if flag is not specified in the '
+        'mark_flag_as_required will pass even if flag is not specified in the '
         'command line!' % flag_name)
-  RegisterValidator(flag_name,
-                    lambda value: value is not None,
-                    message='Flag --%s must be specified.' % flag_name,
-                    flag_values=flag_values)
+  register_validator(flag_name,
+                     lambda value: value is not None,
+                     message='Flag --%s must be specified.' % flag_name,
+                     flag_values=flag_values)
 
 
-def MarkFlagsAsRequired(flag_names, flag_values=FLAGS):
-  """Ensure that flags are not None during program execution.
+def mark_flags_as_required(flag_names, flag_values=FLAGS):
+  """Ensures that flags are not None during program execution.
 
   Recommended usage:
 
     if __name__ == '__main__':
-      MarkFlagsAsRequired(['flag1', 'flag2', 'flag3'])
+      gflags.mark_flags_as_required(['flag1', 'flag2', 'flag3'])
       app.run()
 
   Args:
@@ -240,11 +304,47 @@ def MarkFlagsAsRequired(flag_names, flag_values=FLAGS):
     AttributeError: If any of flag name has not already been defined as a flag.
   """
   for flag_name in flag_names:
-    MarkFlagAsRequired(flag_name, flag_values)
+    mark_flag_as_required(flag_name, flag_values)
 
 
-def _RegisterBoundsValidatorIfNeeded(parser, name, flag_values):
-  """Enforce lower and upper bounds for numeric flags.
+def mark_flags_as_mutual_exclusive(flag_names, required=False,
+                                   flag_values=FLAGS):
+  """Ensures that only one flag among flag_names is set.
+
+  Args:
+    flag_names: [str], a list of the flag names to be checked.
+    required: Boolean, if set, exactly one of the flags must be set.
+        Otherwise, it is also valid for none of the flags to be set.
+    flag_values: An optional FlagValues instance to validate against.
+  """
+
+  def validate_mutual_exclusion(flags_dict):
+    flag_count = sum(1 for val in flags_dict.values() if val is not None)
+    if flag_count == 1 or (not required and flag_count == 0):
+      return True
+    message = ('%s one of (%s) must be specified.' %
+               ('Exactly' if required else 'At most', ', '.join(flag_names)))
+    raise ValidationError(message)
+
+  register_multi_flags_validator(
+      flag_names, validate_mutual_exclusion, flag_values=flag_values)
+
+
+def _add_validator(fv, validator_instance):
+  """Register new flags validator to be checked.
+
+  Args:
+    fv: flagvalues.FlagValues
+    validator_instance: validators.Validator
+  Raises:
+    KeyError: if validators work with a non-existing flag.
+  """
+  for flag_name in validator_instance.get_flags_names():
+    fv[flag_name].validators.append(validator_instance)
+
+
+def _register_bounds_validator_if_needed(parser, name, flag_values):
+  """Enforces lower and upper bounds for numeric flags.
 
   Args:
     parser: NumericParser (either FloatParser or IntegerParser). Provides lower
@@ -254,23 +354,19 @@ def _RegisterBoundsValidatorIfNeeded(parser, name, flag_values):
   """
   if parser.lower_bound is not None or parser.upper_bound is not None:
 
-    def Checker(value):
-      if value is not None and parser.IsOutsideBounds(value):
+    def checker(value):
+      if value is not None and parser.is_outside_bounds(value):
         message = '%s is not %s' % (value, parser.syntactic_help)
-        raise gflags_validators.Error(message)
+        raise ValidationError(message)
       return True
 
-    RegisterValidator(name,
-                      Checker,
-                      flag_values=flag_values)
+    register_validator(name, checker, flag_values=flag_values)
 
 
 # The DEFINE functions are explained in more details in the module doc string.
 
 
-# TODO(vrusinov): 'help' is redefined builtin and should be renamed to helptext
-# (here and in other DEFINE_* functions).
-def DEFINE(parser, name, default, help, flag_values=FLAGS, serializer=None,  # pylint: disable=redefined-builtin
+def DEFINE(parser, name, default, help, flag_values=FLAGS, serializer=None,  # pylint: disable=redefined-builtin,invalid-name
            module_name=None, **args):
   """Registers a generic Flag object.
 
@@ -334,8 +430,8 @@ def DEFINE_flag(flag, flag_values=FLAGS, module_name=None):  # pylint: disable=g
     # pylint: enable=protected-access
 
 
-def _InternalDeclareKeyFlags(flag_names,
-                             flag_values=FLAGS, key_flag_values=None):
+def _internal_declare_key_flags(flag_names,
+                                flag_values=FLAGS, key_flag_values=None):
   """Declares a flag as key for the calling module.
 
   Internal function.  User code should call DECLARE_key_flag or
@@ -393,11 +489,11 @@ def DECLARE_key_flag(  # pylint: disable=g-bad-name
     # These flags are defined in _SPECIAL_FLAGS, and are treated
     # specially during flag parsing, taking precedence over the
     # user-defined flags.
-    _InternalDeclareKeyFlags([flag_name],
-                             flag_values=_helpers.SPECIAL_FLAGS,
-                             key_flag_values=flag_values)
+    _internal_declare_key_flags([flag_name],
+                                flag_values=_helpers.SPECIAL_FLAGS,
+                                key_flag_values=flag_values)
     return
-  _InternalDeclareKeyFlags([flag_name], flag_values=flag_values)
+  _internal_declare_key_flags([flag_name], flag_values=flag_values)
 
 
 def ADOPT_module_key_flags(  # pylint: disable=g-bad-name
@@ -410,28 +506,24 @@ def ADOPT_module_key_flags(  # pylint: disable=g-bad-name
       to be overridden.
 
   Raises:
-    FlagsError: When given an argument that is a module name (a
+    Error: When given an argument that is a module name (a
     string), instead of a module object.
   """
-  # NOTE(salcianu): an even better test would be if not
-  # isinstance(module, types.ModuleType) but I didn't want to import
-  # types for such a tiny use.
-  if isinstance(module, str):
-    raise FlagsError('Received module name %s; expected a module object.'
-                     % module)
+  if not isinstance(module, types.ModuleType):
+    raise Error('Expected a module object, not %r.' % (module,))
   # TODO(vrusinov): _GetKeyFlagsForModule should be public.
-  _InternalDeclareKeyFlags(
+  _internal_declare_key_flags(
       [f.name for f in flag_values._GetKeyFlagsForModule(module.__name__)],  # pylint: disable=protected-access
       flag_values=flag_values)
-  # If module is this flag module, take _SPECIAL_FLAGS into account.
+  # If module is this flag module, take _helpers._SPECIAL_FLAGS into account.
   if module == _helpers.GetModuleObjectAndName(globals())[0]:
-    _InternalDeclareKeyFlags(
+    _internal_declare_key_flags(
         # As we associate flags with _GetCallingModuleObjectAndName(), the
         # special flags defined in this module are incorrectly registered with
         # a different module.  So, we can't use _GetKeyFlagsForModule.
-        # Instead, we take all flags from _SPECIAL_FLAGS (a private
+        # Instead, we take all flags from _helpers._SPECIAL_FLAGS (a private
         # FlagValues, where no other module should register flags).
-        [f.name for f in _helpers.SPECIAL_FLAGS.FlagDict().itervalues()],
+        [f.name for f in six.itervalues(_helpers.SPECIAL_FLAGS.FlagDict())],
         flag_values=_helpers.SPECIAL_FLAGS,
         key_flag_values=flag_values)
 
@@ -451,7 +543,7 @@ def DISCLAIM_key_flags():  # pylint: disable=g-bad-name
   any more flags.  This function will affect all FlagValues objects.
   """
   globals_for_caller = sys._getframe(1).f_globals  # pylint: disable=protected-access
-  module, _ = _GetModuleObjectAndName(globals_for_caller)
+  module, _ = _helpers.GetModuleObjectAndName(globals_for_caller)
   _helpers.disclaim_module_ids.add(id(module))
 
 
@@ -518,7 +610,7 @@ def DEFINE_float(  # pylint: disable=g-bad-name,redefined-builtin
   parser = FloatParser(lower_bound, upper_bound)
   serializer = ArgumentSerializer()
   DEFINE(parser, name, default, help, flag_values, serializer, **args)
-  _RegisterBoundsValidatorIfNeeded(parser, name, flag_values=flag_values)
+  _register_bounds_validator_if_needed(parser, name, flag_values=flag_values)
 
 
 def DEFINE_integer(  # pylint: disable=g-bad-name,redefined-builtin
@@ -541,7 +633,7 @@ def DEFINE_integer(  # pylint: disable=g-bad-name,redefined-builtin
   parser = IntegerParser(lower_bound, upper_bound)
   serializer = ArgumentSerializer()
   DEFINE(parser, name, default, help, flag_values, serializer, **args)
-  _RegisterBoundsValidatorIfNeeded(parser, name, flag_values=flag_values)
+  _register_bounds_validator_if_needed(parser, name, flag_values=flag_values)
 
 
 def DEFINE_enum(  # pylint: disable=g-bad-name,redefined-builtin
@@ -731,7 +823,7 @@ def DEFINE_multi_enum(  # pylint: disable=g-bad-name,redefined-builtin
 
 
 def DEFINE_alias(name, original_name, flag_values=FLAGS, module_name=None):  # pylint: disable=g-bad-name
-  """Define an alias flag for an existing one.
+  """Defines an alias flag for an existing one.
 
   Args:
     name: A string, name of the alias flag.
@@ -741,18 +833,18 @@ def DEFINE_alias(name, original_name, flag_values=FLAGS, module_name=None):  # p
 
   Raises:
     gflags.FlagError:
-      UnrecognizedFlag: if the referenced flag doesn't exist.
-      DuplicateFlag: if the alias name has been used by some existing flag.
+      UnrecognizedFlagError: if the referenced flag doesn't exist.
+      DuplicateFlagError: if the alias name has been used by some existing flag.
   """
   if original_name not in flag_values:
-    raise UnrecognizedFlag('Flag --%s is not defined.' % original_name)
+    raise UnrecognizedFlagError(original_name)
   flag = flag_values[original_name]
 
   class _Parser(ArgumentParser):
     """The parser for the alias flag calls the original flag parser."""
 
-    def Parse(self, argument):
-      flag.Parse(argument)
+    def parse(self, argument):
+      flag.parse(argument)
       return flag.value
 
   class _FlagAlias(Flag):
@@ -784,3 +876,16 @@ DEFINE_string(
     'on the command line even if the program does not define a flag '
     'with that name.  IMPORTANT: flags in this list that have '
     'arguments MUST use the --flag=value format.', _helpers.SPECIAL_FLAGS)
+
+
+# Old CamelCase functions. It's OK to use them, but those use cases will be
+# migrated to PEP8 style functions in the future.
+# pylint: disable=invalid-name
+RegisterValidator = register_validator
+Validator = validator
+RegisterMultiFlagsValidator = register_multi_flags_validator
+MultiFlagsValidator = multi_flags_validator
+MarkFlagAsRequired = mark_flag_as_required
+MarkFlagsAsRequired = mark_flags_as_required
+MarkFlagsAsMutualExclusive = mark_flags_as_mutual_exclusive
+# pylint: enable=invalid-name
